BattleActor = class("BattleActor", function() 
	return cc.Layer:create() 
end)
BattleActor.__index = BattleActor

ACTOR_SIZE = cc.size(240,80)
ACTOR_MIDDLE = 120
ACTOR_SIDE_LEFT = -1
ACTOR_SIDE_RIGHT = 1
--自定义字体类型
FONT_TYPE_DAMAGE     = 1
FONT_TYPE_CRIT       = 2
FONT_TYPE_HP_RECOVER = 3
FONT_TYPE_SP_RECOVER = 4
BattleFontArr = {"ccb/ccbResources/hurtnumber.fnt","ccb/ccbResources/critnumber.fnt","ccb/ccbResources/hpnumber.fnt","ccb/ccbResources/spnumber.fnt"}

--成员变量
BattleActor.type = -1  --  -1为左边 1为右边
BattleActor.actorType = 1   --1：己方英雄 2：对方英雄3：佣兵4：怪物
BattleActor.bNeedSp = false
BattleActor.bDeath = false

BattleActor.maxHp = nil
BattleActor.maxSp = nil
BattleActor.hp    = nil
BattleActor.sp    = nil
BattleActor.name  = nil

BattleActor.hpLabel = nil
BattleActor.hpBar   = nil
BattleActor.spBar   = nil
BattleActor.deathSprite  = nil
BattleActor.headSprite   = nil
BattleActor.hitSprite    = nil
BattleActor.bossSprite   = nil
BattleActor.nameLabel    = nil
BattleActor.stateLayer   = nil
BattleActor.state = {}
BattleActor.round = 0

--攻击动作
function BattleActor:attackAction()
	self:runAction(cc.Sequence:create(cc.MoveBy:create(0.1, cc.p(0-self.type*4,0)), cc.MoveBy:create(0.1, cc.p(self.type*4,0))))
end

--被击动作
function BattleActor:hitAction()
	self:runAction(cc.Sequence:create(cc.MoveBy:create(0.1, cc.p(self.type*4,0)), cc.MoveBy:create(0.1, cc.p(0- self.type*4,0))))
	
	local catch = cc.SpriteFrameCache:getInstance()
    local animFrames = {}
	for j = 1, 5 do
		animFrames[j] = catch:getSpriteFrame("hiteffect_0"..j..".png")
	end

    local animation = cc.Animation:createWithSpriteFrames(animFrames, 0.05)
	self.hitSprite:setVisible(true)
    self.hitSprite:runAction(cc.Sequence:create(cc.Animate:create(animation), cc.CallFunc:create(HideSelf)))
end

--更新血量显示
function BattleActor:updateHp()
	self.hpLabel:setString(math.max(self.hp, 0).."/"..self.maxHp)
	if self.hp <= 0 then
		self.deathSprite:setVisible(true)
	else
		self.deathSprite:setVisible(false)
	end
	self.hpBar:setPercentage(math.max(self.hp, 0) * 100 / self.maxHp)
end

--更新蓝量显示
function BattleActor:updateSp()
	if self.bNeedSp then
		self.spBar:setPercentage(math.max(self.sp, 0) * 100 / self.maxSp)
	end
end

function BattleActor:noteMiss()	
	local label = CreateCommonSprite("miss.png")
	label:setScale(1.5)
	self:addChild(label)
	label:setPosition(Pos(self:getContentSize(),0.3,0.2))
	label:runAction(cc.Sequence:create(cc.DelayTime:create(0.2),cc.MoveBy:create(1, cc.p(0, self:getContentSize().height*0.75)), cc.DelayTime:create(1), cc.CallFunc:create(removeSelf)))

	self:hitAction()
end

--漂字
function BattleActor:note(noteType, value)	
	if BattleFontArr[noteType] == nil then
		return
	end

	local str = ""
	local label = nil
	if value >= 0 then
		str = "+"..value
		label = cc.LabelBMFont:create(str, BattleFontArr[noteType])
	elseif value < 0 then
		str = "-"..math.abs(value)
		label = cc.LabelBMFont:create(str, BattleFontArr[noteType])
	end
	if noteType == FONT_TYPE_CRIT then
		label:setScale(1.8)
	else
		label:setScale(1.5)
	end
	self:addChild(label)
	label:setPosition(Pos(self:getContentSize(),0.3,0.2))
	label:runAction(cc.Sequence:create(cc.DelayTime:create(0.2),cc.MoveBy:create(1, cc.p(0, self:getContentSize().height*0.75)), cc.DelayTime:create(1), cc.CallFunc:create(removeSelf)))
end

function BattleActor:ctor(typeStr, actorType)
	local bNeedSp = false
	if actorType == 1 or actorType == 2 then
		bNeedSp = true
	end
	print("BattleActor:ctor", typeStr, actorType)
	if typeStr == "left" then
		self.type = ACTOR_SIDE_LEFT
	elseif typeStr == "right" then
		self.type = ACTOR_SIDE_RIGHT
	else
		return
	end
	self.actorType = actorType
	
	self:setContentSize(ACTOR_SIZE)
	self:ignoreAnchorPointForPosition(false)

	if bNeedSp then
		self.bNeedSp = true
		--蓝量
		local spBack = CreateCommonSprite("board_sp.png", cc.p(ACTOR_MIDDLE + 3 * self.type, 18))
		-- local spBack = CreateCommonSprite("board_sp.png", cc.p(ACTOR_MIDDLE + 3 * self.type, 10.8))
		self:addChild(spBack)
		self.spBar = CreateCommonBar("pic_sp.png", cc.p(ACTOR_MIDDLE + 3 * self.type, 18))  --
		-- self.spBar = CreateCommonBar("pic_sp.png", cc.p(ACTOR_MIDDLE + 2.8 * self.type, 11.8))  --
		self.spBar:setPercentage(100)
		self:addChild(self.spBar)
		self:addChild(CreateCommonSprite("pic_sp_light.png", cc.p(ACTOR_MIDDLE+ 3 * self.type, 18)))
		-- self:addChild(CreateCommonSprite("pic_sp_light.png", cc.p(ACTOR_MIDDLE + 2.8 * self.type, 11.8)))

		if typeStr == "right" then
			spBack:setFlippedX(true)
			self.spBar:setMidpoint(cc.p(1, 1))
		end
	end

	--血量
	local back = CreateCommonSprite("board_bloodname.png", cc.p(120, 40))
	-- local back = CreateCommonSprite("board_bloodname.png", cc.p(115.8, 41.2))
	-- back:setScaleX(0.88)
	self:addChild(back)
	self.hpBar = CreateCommonBar("pic_blood.png", cc.p(ACTOR_MIDDLE+ 5 * self.type, 40))  --
	-- self.hpBar = CreateCommonBar("pic_blood.png", cc.p(ACTOR_MIDDLE + 3.4 * self.type, 30.6))
	self.hpBar:setPercentage(100)
	self:addChild(self.hpBar)
	self:addChild(CreateCommonSprite("pic_blood_light.png", cc.p(ACTOR_MIDDLE+ 5 * self.type, 43)))
	-- self:addChild(CreateCommonSprite("pic_blood_light.png", cc.p(ACTOR_MIDDLE + 3.4 * self.type, 46)))
	self.hpLabel = CreateCommonLabel("", cc.p(ACTOR_MIDDLE+ 5 * self.type, 40), FontSize1)
	-- self.hpLabel = CreateCommonLabel("", cc.p(ACTOR_MIDDLE + 4.2 * self.type, 30), FontSize1)
	self:addChild(self.hpLabel)
	--死亡
	-- self.deathSprite = CreateCommonSprite("wordboard_dead.png", cc.p(ACTOR_MIDDLE+ 10 * self.type, 35))
	self.deathSprite = CreateCommonFntLabel(ZGMultiLan("battle_value_dead"), FntTypeCommonGlowRed, cc.p(ACTOR_MIDDLE + 10 * self.type, 32))
	-- self.deathSprite:setColor(COLOR_BLACK)
	self.deathSprite:setVisible(false)
	self:addChild(self.deathSprite)

	--头像
	local headScale = 0.7
	self.headSprite = CreateCommonSprite("board_head.png", cc.p(ACTOR_MIDDLE+ 130 * self.type, 40))
	self.headSprite:setScale(headScale)
	self:addChild(self.headSprite)
	self.hitSprite = CreateCommonSprite("hiteffect_05.png", cc.p(ACTOR_MIDDLE + 8 + 130 * self.type, 40))
	self.hitSprite:setVisible(false)
	self:addChild(self.hitSprite)

	local headFrame = CreateCommonSprite("frame_head.png", cc.p(ACTOR_MIDDLE+ 130 * self.type, 40))
	headFrame:setScale(headScale)
	self:addChild(headFrame)
	self.bossSprite = CreateCommonSprite("wordboard_boss.png", Pos(headFrame:getContentSize(),0.5,0.13))
	self.bossSprite:setVisible(false)
	headFrame:addChild(self.bossSprite)

	--名字
	self.nameLabel = CreateCommonLabel("name", cc.p(ACTOR_MIDDLE + 75 * self.type, 67), FontSize2)
	-- self.nameLabel = CreateCommonLabel("name", cc.p(ACTOR_MIDDLE + 101.8 * self.type, 54.2), FontSize1)
	self.nameLabel:setAnchorPoint(cc.p(0.5 + 0.5*self.type,0.5))
	self:addChild(self.nameLabel) 

	--状态
	self.stateLayer = cc.Layer:create()
	if bNeedSp then
		self.stateLayer:setPosition(cc.p(ACTOR_MIDDLE+ 80 * self.type, -5))
	else
		self.stateLayer:setPosition(cc.p(ACTOR_MIDDLE+ 80 * self.type, 8))
	end
	self.stateLayer:setAnchorPoint(cc.p(0.5 + 0.5 * self.type,0.5))
	self:addChild(self.stateLayer) 
	self.state = {}

	if typeStr == "right" then
		tolua.cast(back, "cc.Sprite"):setFlippedX(true)
		self.hpBar:setMidpoint(cc.p(1, 1))
	end
end

--设置最大血量
function BattleActor:setMaxHp(hpValue)
	print("setMaxHp", hpValue)
	self.maxHp = hpValue
	if self.hp == nil then
		self.hp = self.maxHp
		self:updateHp()
	end
end

--设置最大蓝量
function BattleActor:setMaxSp(spValue)
	print("setMaxSp", spValue)
	self.maxSp = spValue
	if self.sp == nil then
		self.sp = self.maxSp
		self:updateSp()
	end
end

--血量变化
function BattleActor:hpChange(changeValue,bCrit)
	self.hp = self.hp + changeValue
	
	if changeValue >= 0 then
		self:note(FONT_TYPE_HP_RECOVER, changeValue)
	else
		if bCrit then
			self:note(FONT_TYPE_CRIT, changeValue)
		else
			self:note(FONT_TYPE_DAMAGE, changeValue)
		end
		
		self:hitAction()
	end

	self:updateHp()
end

--蓝量变化
function BattleActor:spChange(changeValue)
	print("spChange",changeValue)
	self.sp = self.sp + changeValue
	
	if changeValue > 0 then
		self:note(FONT_TYPE_SP_RECOVER, changeValue)
	end

	self:updateSp()
end

--设置等级和名字
function BattleActor:setLevelAndName(level,name)
	local str = nil
	self.name = name
	if self.type == ACTOR_SIDE_LEFT then
		str = name.."Lv."..level
	elseif self.type == ACTOR_SIDE_RIGHT then
		str = "Lv."..level..name
	end
	self.nameLabel:setString(str)
	--[[颜色
	local colorArr = {COLOR_PURPLE, COLOR_GRAY, COLOR_PURPLE, COLOR_GRAY}
	self.nameLabel:setColor(colorArr[self.actorType])
	]]
end

--设置Boss
function BattleActor:setActorBoss()
	self.bossSprite:setVisible(true)
end

--设置头像
function BattleActor:setHead(headSpriteName)
	self.headSprite:setSpriteFrame(cc.SpriteFrameCache:getInstance():getSpriteFrame(headSpriteName))
end

--使用技能 消耗mp
function BattleActor:useSkill(mpCost)
	if self.bNeedSp then
		self.sp = self.sp - mpCost
		self:updateSp()
	end
end

--添加状态
function BattleActor:AddState(state, round)
	local stateName = {MultiLan("BA_buffName_1"),MultiLan("BA_buffName_2"),MultiLan("BA_buffName_3"),MultiLan("BA_buffName_4"),MultiLan("BA_buffName_5"), 
					   MultiLan("BA_buffName_6"),MultiLan("BA_buffName_7"),MultiLan("BA_buffName_8"), nil   , nil   ,
					   MultiLan("BA_buffName_11"),MultiLan("BA_buffName_12"),MultiLan("BA_buffName_13"),MultiLan("BA_buffName_14"),MultiLan("BA_buffName_15"), 
					   MultiLan("BA_buffName_16"),MultiLan("BA_buffName_17"),MultiLan("BA_buffName_18"),MultiLan("BA_buffName_19")}
	local findStateInfo = FindFirstByKey(self.state,"state",state)

	if findStateInfo == nil then
		table.insert(self.state, {state = state,name= stateName[state], round = round, bPre= false})--GameData:getBuff(state).name
		if state == 16 or state == 17 then
			self.state[table.maxn(self.state)].bPre = true
		end
	elseif round >= findStateInfo.round then
		findStateInfo.round = round
	end
end

--减少buff回合
function BattleActor:ReduceRound(round)
	print("ReduceRound",round,self.round)
	for i=1, table.maxn(self.state) do
		--print(i, self.state[i], self.state[i].round, self.state[i].state, self.state[i].bPre)
		if self.state[i].bPre == false and round > self.round then
			self.state[i].round = self.state[i].round - 1
		end
	end
	self.round = round
	while BattleActorTest(self.state) == false do
	
	end
	
	self:updateState()
end

--是否不需要继续查找round=0
function BattleActorTest(t)
	local bOk = true
	for i=1,table.maxn(t) do
		if t[i].round <= 0 then
			table.remove(t,i)
			bOk = false
			break
		end
	end

	return bOk
end

--删除状态
function BattleActor:deleteState(buffId)
	for i=1, table.maxn(self.state) do
		if self.state[i].state == buffId then
			self.state[i].round = self.state[i].round - 1
		end
	end
	
	while BattleActorTest(self.state)== false do
	
	end
	
	self:updateState()
end

--更新状态显示
function BattleActor:updateState()
	self.stateLayer:removeAllChildren(true)
	
	local richText = ccui.RichText:create()
	richText:ignoreContentAdaptWithSize(false)
	if self.type == ACTOR_SIDE_LEFT then
		richText:setAnchorPoint(cc.p(0 , 0.5))
	elseif self.type == ACTOR_SIDE_RIGHT then
		richText:setAnchorPoint(cc.p(0.5 , 0.5))
	end
	
	self.stateLayer:addChild(richText)
	
	local width = 0
	for i=1, table.maxn(self.state) do
		if self.state[i].round > 0 then
			richText:pushBackElement(ccui.RichElementText:create(3*(i-1)+1, COLOR_WHITE, 255,"[",DefaultFontName,FontSize1))
			if self.state[i].state <= 10 then
				richText:pushBackElement(ccui.RichElementText:create(3*(i-1)+2, COLOR_GREEN, 255,self.state[i].name..self.state[i].round, DefaultFontName, FontSize1))
			else
				richText:pushBackElement(ccui.RichElementText:create(3*(i-1)+2, COLOR_RED, 255,self.state[i].name..self.state[i].round, DefaultFontName, FontSize1))
			end
			richText:pushBackElement(ccui.RichElementText:create(3*(i-1)+3, COLOR_WHITE, 255,"]",DefaultFontName,FontSize1))
			width = width + GetWidth("["..self.state[i].name..self.state[i].round.."]", FontSize1)
		end
	end
	
    richText:setContentSize(cc.size(width, 40))
	richText:setPosition(cc.p(0-width/2, 0))
end

return BattleActor