EquipHelper = class("EquipHelper")
EquipHelper.__index = EquipHelper

--获取装备静态数据
function EquipHelper:getStaticData(infoTable)
	if infoTable.sData == nil then
		infoTable.sData = GameData:getPropDataTable(infoTable.prop_id)
	end
		
	return infoTable.sData 
end

--获取评分
function EquipHelper:getEquipScore(infoTable)
	if infoTable.score == nil then
		return UserAttr:CalEquipScore(infoTable)
	else
		return infoTable.score
	end
end

--获得装备颜色  参数为服务器的装备属性的extra字段
function EquipHelper:getEquipColor(infoTable)
	if infoTable.grade ~= nil then
		return EQUIP_COLOR_ARR[infoTable.grade],infoTable.grade
	else
		local grade = EquipHelper:getEquipGrade(infoTable)
		return EQUIP_COLOR_ARR[grade],grade
	end
end

--获得装备品阶  参数为服务器的装备属性的extra字段
function EquipHelper:getEquipGrade(infoTable)
	local grade = 1
	if infoTable.extra ~= nil and infoTable.extra ~= CJSON_NULL then
		table.foreach(infoTable.extra,function(i, v) grade = grade+1 end)
	end
	print("EquipHelper:getEquipGrade(infoTable)",grade)
	return grade
end

function EquipHelper:getEquipAttrNumber(keyStr, value, infoTable)
	local retNumber = value
	local retString = MultiLan("attr_role_" .. keyStr)

	retNumber = math.ceil(retNumber * (1 + getStrengthPercent(infoTable)/100))
	return retNumber, retString
end

--创建装备基本属性信息richtext   equipShowColor表示特殊神器打造界面的显示  bChangedEquip表示被换的装备不显示多余数据
function EquipHelper:CreateEquipDesc(infoTable)
	local infoText = CreateLayer(cc.size(270, 500))
	local propInfoTable = self:getStaticData(infoTable)
	local realHeight = 0
	local y = 500
    
    local equipColor = self:getEquipColor(infoTable)

	local strengthenStr = ""
	if infoTable.strengthen  ~= nil and infoTable.strengthen > 0 then
		strengthenStr = " +" .. infoTable.strengthen
	end
	local desc = CreateCommonLabel("Lv" .. propInfoTable.level .. " " .. MultiLan(propInfoTable.name) .. strengthenStr,
										 cc.p(0,y), FontSize4, equipColor, cc.p(0,1))
	desc:disableEffect()
	y = y - desc:getContentSize().height
	infoText:addChild(desc)
	
	if infoTable.obj ~= nil and infoTable.obj ~= 0 then
		local who = ""
		if infoTable.obj == 1 then
			who = UserSelfInfo.name
		else
			who = MultiLan(GameData:getRoleDataById(infoTable.obj).name)
		end
	
		local equipLabel = CreateCommonLabel(MultiLan("lan_equiped").." (" .. who .. ")", cc.p(0,y), FontSize2, nil, cc.p(0,1))
		equipLabel:disableEffect()
		y = y - equipLabel:getContentSize().height
		infoText:addChild(equipLabel)
	end

	if propInfoTable.role ~= 0 then
		desc = CreateCommonLabel(string.format(MultiLan("EH_equipCarrer"), MultiLan("role_career_" .. propInfoTable.role)),
										 cc.p(0,y), FontSize2, COLOR_YELLOW, cc.p(0,1))
		desc:disableEffect()
		y = y - desc:getContentSize().height
		infoText:addChild(desc)
	end

	--评分
	local singleHeight = 0
	desc = CreateCommonLabel(MultiLan("EH_equipScore")..EquipHelper:getEquipScore(infoTable),cc.p(0,y), FontSize2, EQUIP_COLOR_ARR[4], cc.p(0,1))
	y = y - desc:getContentSize().height
	desc:disableEffect()
	infoText:addChild(desc)
	singleHeight = desc:getContentSize().height


	--主属性
	for k,v in pairs(infoTable.attr) do
		local equipAttrNumber, equipAttrString = self:getEquipAttrNumber(k,v,infoTable)
		if k == "damage" then
			equipAttrNumber = " "..math.ceil(equipAttrNumber/2).." - "..equipAttrNumber
		else
			equipAttrNumber = " +"..equipAttrNumber
		end
		desc = CreateCommonLabel(equipAttrString .. equipAttrNumber,cc.p(0,y), FontSize4, nil, cc.p(0,1))
		y = y - desc:getContentSize().height
		desc:disableEffect()
		infoText:addChild(desc)
	end

	--附加属性
	local extraTable = {"str", "int", "agi", "sta"}
	local str = ""
	local n = 0
	if infoTable.extra ~= nil and infoTable.extra ~= CJSON_NULL then
		for i=1, table.maxn(MAIN_ATTR_DESC) do
			if infoTable.extra[extraTable[i]] ~= nil and infoTable.extra[extraTable[i]] ~= CJSON_NULL then
				n = n + 1
				str = str..MAIN_ATTR_DESC[i]..": "..infoTable.extra[extraTable[i]].."\n"
			end
		end
	end
	if n ~= 0 then
		desc = CreateCommonLabel(str, cc.p(0,y), FontSize2, equipColor, cc.p(0,1), cc.size(270,(singleHeight+1)*n))
		--desc:setDe
		desc:disableEffect()
		infoText:addChild(desc)
		y = y - (singleHeight+1)*n
	end
	
	--神器属性
	if EquipHelper:isMaster(infoTable) then
		desc = CreateCommonLabel(infoTable.master_level .. MultiLan("EH_artifactStar"),cc.p(0,y), FontSize2, COLOR_RED, cc.p(0,1))
		y = y - desc:getContentSize().height
		desc:disableEffect()
		infoText:addChild(desc)

		for k, v in pairs(infoTable.master) do
			if k == "arp" or  k == "prp" or  k == "mrp" then
				desc = CreateCommonLabel(MASTER_ATTR_DESC[k] .. " +" .. v * infoTable.master_level,cc.p(0,y), FontSize2, COLOR_RED, cc.p(0,1))
			else
				desc = CreateCommonLabel(MASTER_ATTR_DESC[k] .. " +" .. (v * infoTable.master_level*100).."%",cc.p(0,y), FontSize2, COLOR_RED, cc.p(0,1))
			end
			y = y - desc:getContentSize().height
			desc:disableEffect()
			infoText:addChild(desc)
		end
	end
	
	--雕文属性
	if EquipHelper:isGlyph(infoTable) then
		desc = CreateCommonLabel(infoTable.glyph_level .. MultiLan("EH_GlyphStar"), cc.p(0 ,y), FontSize2, COLOR_RED, cc.p(0,1))
		y = y - desc:getContentSize().height
		desc:disableEffect()
		infoText:addChild(desc)

		for k, v in pairs(infoTable.glyph) do
			if k == "arp" or  k == "prp" or  k == "mrp" then
				desc = CreateCommonLabel(MASTER_ATTR_DESC[k] .. " +" .. v * infoTable.glyph_level,cc.p(0,y), FontSize2, COLOR_RED, cc.p(0,1))
			else
				desc = CreateCommonLabel(MASTER_ATTR_DESC[k] .. " +" .. (v * infoTable.glyph_level*100).."%",cc.p(0,y), FontSize2, COLOR_RED, cc.p(0,1))
			end
			y = y - desc:getContentSize().height
			desc:disableEffect()
			infoText:addChild(desc)
		end
	end
	
	realHeight = 500 - y
	return infoText, realHeight
end

--创建装备基本属性信息richtext   equipShowColor表示特殊神器打造界面的显示  bChangedEquip表示被换的装备不显示多余数据 bShowOthers表示显示其他人装备
function EquipHelper:doCreateItemInfoRichtext(infoTable, equipShowColor, bChangedEquip, bShowOthers)
	local infoText = ccui.RichText:create()
	infoText:setContentSize(cc.size(270, 500))
	infoText:ignoreContentAdaptWithSize(false)
	print("EquipHelper",infoTable.prop_id)
	local propInfoTable = self:getStaticData(infoTable)
    
	print(1)
    if propInfoTable.ptype ~= 7 then --物品
       	--物品描述
	    if propInfoTable.desc ~= nil and propInfoTable.desc ~= CJSON_NULL then
		    infoText:pushBackElement(ccui.RichElementText:create(1, COLOR_WHITE, 255, "  "..MultiLan(propInfoTable.desc), DefaultFontName, 22))
		    infoText:pushBackElement(ccui.RichElementNewLine:create())
	    end

       return infoText
    end
    local equipColor =  nil
    if equipShowColor ~= nil then
       equipColor = equipShowColor
    else
       equipColor = EquipHelper:getEquipColor(infoTable)
    end
	print(2,propInfoTable.level, propInfoTable.level, propInfoTable.name, propInfoTable.name)
	if propInfoTable.level ~= nil and propInfoTable.level ~= CJSON_NULL and propInfoTable.name ~= nil and propInfoTable.name ~= CJSON_NULL then
		local strengthenStr = ""
		if infoTable.strengthen  ~= nil and infoTable.strengthen > 0 then
			strengthenStr = " +" .. infoTable.strengthen
		end
		infoText:pushBackElement(ccui.RichElementText:create(1, equipColor, 255, "Lv" .. propInfoTable.level .. " " .. MultiLan(propInfoTable.name) .. strengthenStr, DefaultFontName, FontSize4))
		infoText:pushBackElement(ccui.RichElementNewLine:create())
		
		if infoTable.obj ~= nil and infoTable.obj ~= 0 and (bShowOthers == nil or bShowOthers == false) then
			local who = ""
			if infoTable.obj == 1 then
				who = UserSelfInfo.name
			else
				who = MultiLan(GameData:getRoleDataById(infoTable.obj).name)
			end
			infoText:pushBackElement(ccui.RichElementText:create(1, COLOR_WHITE, 255, MultiLan("lan_equiped").." (" .. who .. ")", DefaultFontName, FontSize2))
		end
		infoText:pushBackElement(ccui.RichElementNewLine:create())
	end

	if propInfoTable.role ~= 0 then
		infoText:pushBackElement(ccui.RichElementText:create(1, COLOR_YELLOW, 255, string.format(MultiLan("EH_equipCarrer"), MultiLan("role_career_" .. propInfoTable.role)), DefaultFontName, 22))
		infoText:pushBackElement(ccui.RichElementNewLine:create())
	end

	--评分
	if equipShowColor == nil then
		infoText:pushBackElement(ccui.RichElementText:create(1, EQUIP_COLOR_ARR[4], 255, MultiLan("EH_equipScore")..EquipHelper:getEquipScore(infoTable), DefaultFontName, 22))
		infoText:pushBackElement(ccui.RichElementNewLine:create())
	end

	--主属性
    if equipShowColor ~= nil then
        local equipAttrString = MultiLan( "attr_type_equip_"..propInfoTable.attr_type)
        local equipAttrNumber = propInfoTable.attr_value
        if propInfoTable.attr_type == 1 then
			equipAttrNumber = " "..math.ceil(equipAttrNumber/2).." - "..equipAttrNumber
		 else
			equipAttrNumber = " +"..equipAttrNumber
		end
        if equipAttrString == nil then
          equipAttrString = MultiLan("EH_mainProperty")
        end
       	infoText:pushBackElement(ccui.RichElementText:create(1, COLOR_WHITE, 255, equipAttrString ..equipAttrNumber, DefaultFontName, FontSize4))
		infoText:pushBackElement(ccui.RichElementNewLine:create())
    else
	    if infoTable.attr ~= nil and infoTable.attr ~= CJSON_NULL then
		    for k,v in pairs(infoTable.attr) do
			    local equipAttrNumber, equipAttrString = self:getEquipAttrNumber(k,v,infoTable)
			    if k == "damage" then
				    equipAttrNumber = " "..math.ceil(equipAttrNumber/2).." - "..equipAttrNumber
			    else
				    equipAttrNumber = " +"..equipAttrNumber
			    end
			
			    infoText:pushBackElement(ccui.RichElementText:create(1, COLOR_WHITE, 255, equipAttrString .. equipAttrNumber, DefaultFontName, FontSize4))
			    infoText:pushBackElement(ccui.RichElementNewLine:create())
		    end
	    end
    end

	--附加属性
	local extraTable = {"str", "int", "agi", "sta"}
	if infoTable.extra ~= nil and infoTable.extra ~= CJSON_NULL then
		for i=1, table.maxn(MAIN_ATTR_DESC) do
			if infoTable.extra[extraTable[i]] ~= nil and infoTable.extra[extraTable[i]] ~= CJSON_NULL then
				infoText:pushBackElement(ccui.RichElementText:create(1, equipColor, 255, MAIN_ATTR_DESC[i] .. ": "..infoTable.extra[extraTable[i]], DefaultFontName,22))
				infoText:pushBackElement(ccui.RichElementNewLine:create())
			end
		end
	end

	--神器属性
	if EquipHelper:isMaster(infoTable) then
        if infoTable.master_level == nil then
            infoTable.master_level = 1
        end
		local alllevel = 0
		print("alllevel", infoTable.alllevel)
		if infoTable.alllevel ~= nil then
			alllevel = infoTable.alllevel
		else
			alllevel = UserAttr:getAllLevel()
		end
		
		local activeLevel = math.min(infoTable.master_level,alllevel)
		print("activeLevel", activeLevel, infoTable.master_level, alllevel)
        if equipShowColor == nil then
            infoText:pushBackElement(ccui.RichElementText:create(1, COLOR_RED, 255, infoTable.master_level .. MultiLan("EH_artifactStar"), DefaultFontName, 22))
		    infoText:pushBackElement(ccui.RichElementNewLine:create())
        end
		for k, v in pairs(infoTable.master) do
			if k == "arp" or  k == "prp" or  k == "mrp" then
				infoText:pushBackElement(ccui.RichElementText:create(1, COLOR_RED, 255, MASTER_ATTR_DESC[k] .. " +" .. v * infoTable.master_level, DefaultFontName, 22))
			else
				infoText:pushBackElement(ccui.RichElementText:create(1, COLOR_RED, 255, MASTER_ATTR_DESC[k] .. " +" .. (v * infoTable.master_level*100).."%", DefaultFontName, 22))
			end
			infoText:pushBackElement(ccui.RichElementNewLine:create())
		end
		
		if bChangedEquip then

		else
			--激活神器属性
			if infoTable.obj ~= nil and infoTable.obj == 1 and activeLevel > 0 then
				local activeStr = "["..string.format(MultiLan("EH_artifactActive_1"),tostring(activeLevel)).."]"
				for k, v in pairs(infoTable.master) do
					if k == "arp" or  k == "prp" or  k == "mrp" then
						infoText:pushBackElement(ccui.RichElementText:create(1, COLOR_GREEN, 255, MASTER_ATTR_DESC[k] .. " +" ..(v * activeLevel)..activeStr, DefaultFontName, 22))
					else
						infoText:pushBackElement(ccui.RichElementText:create(1, COLOR_GREEN, 255, MASTER_ATTR_DESC[k] .. " +" .. (v * activeLevel*100).."%"..activeStr, DefaultFontName, 22))
					end
					infoText:pushBackElement(ccui.RichElementNewLine:create())
				end
			end
			
			--未激活神器属性
			if infoTable.obj ~= nil and infoTable.obj == 1 then
				if alllevel >= infoTable.master_level then
					infoText:pushBackElement(ccui.RichElementText:create(1, EQUIP_COLOR_ARR[1], 255, MultiLan("EH_artifactActive_2"), DefaultFontName, 22))
					infoText:pushBackElement(ccui.RichElementNewLine:create())
				else
					activeLevel = activeLevel + 1
					local activeStr = string.format(MultiLan("EH_artifactActive_3"),tostring(activeLevel))
					for k, v in pairs(infoTable.master) do
						if k == "arp" or  k == "prp" or  k == "mrp" then
							infoText:pushBackElement(ccui.RichElementText:create(1, EQUIP_COLOR_ARR[1], 255, MASTER_ATTR_DESC[k] .. " +" ..(v * activeLevel)..activeStr, DefaultFontName, 22))
						else
							infoText:pushBackElement(ccui.RichElementText:create(1, EQUIP_COLOR_ARR[1], 255, MASTER_ATTR_DESC[k] .. " +" .. (v * activeLevel*100).."%"..activeStr, DefaultFontName, 22))
						end
						infoText:pushBackElement(ccui.RichElementNewLine:create())
					end
				end
			end
		end
	end

	--雕纹属性
	if  EquipHelper:isGlyph(infoTable)  then
        if infoTable.glyph_level == nil then
            infoTable.glyph_level = 1
        end
		local alllevel = 0
		print("alllevel", infoTable.alllevel)
		if infoTable.alllevel ~= nil then
			alllevel = infoTable.alllevel
		else
			alllevel = UserAttr:getAllLevel()
		end
		
		local activeLevel = math.min(infoTable.glyph_level,alllevel)
		print("activeLevel", activeLevel, infoTable.glyph_level, alllevel)
        if equipShowColor == nil then
            infoText:pushBackElement(ccui.RichElementText:create(1, COLOR_RED, 255, infoTable.glyph_level .. MultiLan("EH_GlyphStar"), DefaultFontName, 22))
		    infoText:pushBackElement(ccui.RichElementNewLine:create())
        end
		for k, v in pairs(infoTable.glyph) do
			if k == "arp" or  k == "prp" or  k == "mrp" then
				infoText:pushBackElement(ccui.RichElementText:create(1, COLOR_RED, 255, MASTER_ATTR_DESC[k] .. " +" .. v * infoTable.glyph_level, DefaultFontName, 22))
			else
				infoText:pushBackElement(ccui.RichElementText:create(1, COLOR_RED, 255, MASTER_ATTR_DESC[k] .. " +" .. (v * infoTable.glyph_level*100).."%", DefaultFontName, 22))
			end
			infoText:pushBackElement(ccui.RichElementNewLine:create())
		end
		
		if bChangedEquip then
			return infoText
		end
		
		--激活神器属性
		if infoTable.obj ~= nil and infoTable.obj == 1 and activeLevel > 0 then
			local activeStr = "["..string.format(MultiLan("EH_artifactActive_1"),tostring(activeLevel)).."]"
			for k, v in pairs(infoTable.glyph) do
				if k == "arp" or  k == "prp" or  k == "mrp" then
					infoText:pushBackElement(ccui.RichElementText:create(1, COLOR_GREEN, 255, MASTER_ATTR_DESC[k] .. " +" ..(v * activeLevel)..activeStr, DefaultFontName, 22))
				else
					infoText:pushBackElement(ccui.RichElementText:create(1, COLOR_GREEN, 255, MASTER_ATTR_DESC[k] .. " +" .. (v * activeLevel*100).."%"..activeStr, DefaultFontName, 22))
				end
				infoText:pushBackElement(ccui.RichElementNewLine:create())
			end
		end
		
		--未激活神器属性
		if infoTable.obj ~= nil and infoTable.obj == 1 then
			if alllevel >= infoTable.glyph_level then
				infoText:pushBackElement(ccui.RichElementText:create(1, EQUIP_COLOR_ARR[1], 255, MultiLan("EH_artifactActive_2"), DefaultFontName, 22))
				infoText:pushBackElement(ccui.RichElementNewLine:create())
			else
				activeLevel = activeLevel + 1
				local activeStr = string.format(MultiLan("EH_artifactActive_3"),tostring(activeLevel))
				for k, v in pairs(infoTable.glyph) do
					if k == "arp" or  k == "prp" or  k == "mrp" then
						infoText:pushBackElement(ccui.RichElementText:create(1, EQUIP_COLOR_ARR[1], 255, MASTER_ATTR_DESC[k] .. " +" ..(v * activeLevel)..activeStr, DefaultFontName, 22))
					else
						infoText:pushBackElement(ccui.RichElementText:create(1, EQUIP_COLOR_ARR[1], 255, MASTER_ATTR_DESC[k] .. " +" .. (v * activeLevel*100).."%"..activeStr, DefaultFontName, 22))
					end
					infoText:pushBackElement(ccui.RichElementNewLine:create())
				end
			end
		end
	end
		
	if bChangedEquip then
		return infoText
	end

	if infoTable.stone ~= nil and infoTable.stone ~= CJSON_NULL then
		for i = 1, table.maxn(infoTable.stone) do
			local attrTable = {"agi","str","int","sta"}
			local stoneInfo = GameData:getPropDataTable(infoTable.stone[i])
			local stoneString = MultiLan(stoneInfo.name) .. " " .. ATTR_DESC[attrTable[stoneInfo.place]] .. " +" .. stoneInfo.attr_value
			infoText:pushBackElement(ccui.RichElementText:create(1, COLOR_PURPLE, 255, stoneString, DefaultFontName, 22))
			infoText:pushBackElement(ccui.RichElementNewLine:create())
		end
	end
	return infoText
end

function EquipHelper:createItemInfoRichtext(infoTable, equipShowColor, bChangedEquip, bShowOthers)
    return  EquipHelper:doCreateItemInfoRichtext(infoTable, equipShowColor, bChangedEquip, bShowOthers)
end

function EquipHelper:setEquipPlaceNameLabel(fatherLayer,iteminfo)
	local place = iteminfo.place
	if place == nil and self:getStaticData(iteminfo) ~= nil then
		place = self:getStaticData(iteminfo).place
	end
	
	if place ~= nil then
		fatherLayer:addChild(CreateCommonLabel(MultiLan("equip_place_" .. place),Pos(fatherLayer:getContentSize(),0.95,0.95),FontSize4,COLOR_ORANGE,cc.p(1,1)))
	end
end
--是否红装
function EquipHelper:isRedEquip(equipInfo)
	if (equipInfo.master ~= nil and equipInfo.master ~= CJSON_NULL) or (equipInfo.glyph ~= nil and equipInfo.glyph ~= CJSON_NULL) then
		return true
	end
	
	return false
end
--是否神器
function EquipHelper:isMaster(equipInfo)
	if equipInfo.master ~= nil and equipInfo.master ~= CJSON_NULL then
		return true
	end
	
	return false
end
--是否雕纹神器
function EquipHelper:isGlyph(equipInfo)
	if equipInfo.glyph ~= nil and equipInfo.glyph ~= CJSON_NULL then
		return true
	end
	
	return false
end
--是否只是神器
function EquipHelper:isOnlyMaster(equipInfo)
	if (equipInfo.master ~= nil and equipInfo.master ~= CJSON_NULL) and (equipInfo.glyph ~= nil and equipInfo.glyph == CJSON_NULL) then
		return true
	end
	
	return false
end
--是否只是雕文
function EquipHelper:isOnlyGlyph(equipInfo)
	if (equipInfo.master ~= nil and equipInfo.master == CJSON_NULL) and (equipInfo.glyph ~= nil and equipInfo.glyph ~= CJSON_NULL) then
		return true
	end
	
	return false
end
--是否既是神器也是雕文
function EquipHelper:isBothMG(equipInfo)
	if (equipInfo.master ~= nil and equipInfo.master ~= CJSON_NULL) and (equipInfo.glyph ~= nil and equipInfo.glyph ~= CJSON_NULL) then
		return true
	end
	
	return false
end
